#!/bin/sh
PE_LEVEL=pe1309
# IBM_PROLOG_BEGIN_TAG 
# This is an automatically generated prolog. 
#  
#  
#  
# Licensed Materials - Property of IBM 
#  
# (C) COPYRIGHT International Business Machines Corp. 2013,2014 
# All Rights Reserved 
#  
# US Government Users Restricted Rights - Use, duplication or 
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp. 
#  
# IBM_PROLOG_END_TAG 
PERTE_TAR_FN=pe_rte.tar

SCRIPT_NAME=$0

function STDOUT {
    echo $* >&1 
}

function STDERR {
    echo $* >&2 
}

function PRINT_ERROR {
# Print error to stderr
    STDERR "${SCRIPT_NAME}: ERROR "$*
}

usage () {
  echo "USAGE: $0 [-ci] <mic list>"
  echo "       Extract PE RTE context from RPM and install them onto MIC nodes"
  echo "       If <mic list> is not provided, only extraction will be done. "
  echo "       Options:"
  echo "            -c: Perform installation of compiler script support for MIC on the host node"
  echo "            -i: Perform installation using the existing $PERTE_TAR_FN"
  exit 1;
}

# INSTALL_ONLY won't do rpm extraction and tar.
# The tar file must exist prior
INSTALL_ONLY=0
INSTALL_COMPILER=0

while getopts ci arg
do
  case $arg in
    c) INSTALL_COMPILER=1
       ;;
    i) INSTALL_ONLY=1
       if [[ ! -e $PERTE_TAR_FN ]]; then
          PRINT_ERROR " $PERTE_TAR_FN does not exist"
          exit 1
       fi ;;
    \?) usage $0
        exit 1;;
  esac
done

shift $(($OPTIND - 1))

MIC_LIST=$*

if [[ $INSTALL_ONLY -eq 0 ]]; then
  if [[ -d ./opt ]]; then
     STDOUT "Removing old ./opt"
    /bin/rm -rf ./opt
  fi
  if [[ -e ./$PERTE_TAR_FN ]]; then
     STDOUT "Removing old $PERTE_TAR_FN"
    /bin/rm -f $PERTE_TAR_FN
  fi
  STDOUT "Extracting RPM contents ..."
  rpm2cpio ppe*.knc.rpm | cpio -idm

  STDOUT "Adding PE license ..."
  LICENSE_DIR=`ls /opt/ibmhpc/ | grep "pe..00"`
  if [[ -z ${LICENSE_DIR} ]]; then
      STDERR "No PE RT license found on hosting node"
      exit 1
  fi
  if [[ ! -d ./opt/ibmhpc/${LICENSE_DIR}/lap/ ]]; then
    mkdir -p ./opt/ibmhpc/${LICENSE_DIR}/lap/
  fi
  cp -r /opt/ibmhpc/${LICENSE_DIR}/lap/license ./opt/ibmhpc/${LICENSE_DIR}/lap/

  STDOUT "Creating tar file ..."
  tar -cf $PERTE_TAR_FN ./opt
fi

if [[ $INSTALL_COMPILER -ne 0 ]]; then
  STDOUT "Install compiler script support for MIC"
  TARGET_DIR=/opt/ibmhpc/${PE_LEVEL}/mic
  mkdir -p ${TARGET_DIR}/include64
  mkdir -p ${TARGET_DIR}/lib64
  TARGET_DIR=${TARGET_DIR}/lib64

  # copy fast MPICH2 libraries
  cp ./opt/ibmhpc/${PE_LEVEL}/mpich2/intel/fast/lib64/libmpigc4.so ${TARGET_DIR}/.
  cp ./opt/ibmhpc/${PE_LEVEL}/mpich2/intel/fast/lib64/libmpigf.so ${TARGET_DIR}/.
  cp ./opt/ibmhpc/${PE_LEVEL}/mpich2/intel/fast/lib64/libmpi.so ${TARGET_DIR}/.
  cp ./opt/ibmhpc/${PE_LEVEL}/mpich2/intel/fast/lib64/libmpl.so ${TARGET_DIR}/.
  cp ./opt/ibmhpc/${PE_LEVEL}/mpich2/intel/fast/lib64/libopa.so ${TARGET_DIR}/.

  # copy PE libraries
  cp ./opt/ibmhpc/${PE_LEVEL}/ppe.poe/lib64/libGlobalWtime_realtime.so ${TARGET_DIR}/.
  cp ./opt/ibmhpc/${PE_LEVEL}/ppe.poe/lib64/libGlobalWtime.so ${TARGET_DIR}/.
  cp ./opt/ibmhpc/${PE_LEVEL}/ppe.pami/intel/lib64/pami64/libpami.so ${TARGET_DIR}/.
  cp ./opt/ibmhpc/${PE_LEVEL}/ppe.pami/intel/lib64/pami64/libpamiudp.so ${TARGET_DIR}/.
  cp ./opt/ibmhpc/${PE_LEVEL}/hal/lib64/ib/libhal64_ibm.so ${TARGET_DIR}/.
  cp ./opt/ibmhpc/${PE_LEVEL}/pempi/gnu/lib64/libpoe.so ${TARGET_DIR}/.
  cp ./opt/ibmhpc/${PE_LEVEL}/ppe.openshmem/gnu/lib64/libshmem.so ${TARGET_DIR}/.
fi

for MIC in $MIC_LIST; do
  STDOUT "SCP tar file to $MIC"
  scp $PERTE_TAR_FN root@$MIC:/tmp

  STDOUT "SSH: untar $PERTE_TAR_FN on $MIC" 
  ssh root@$MIC "cd /tmp; tar -xf ./pe_rte.tar -C /"

  STDOUT "SSH: Run pe_install"
  ssh root@$MIC "PE_LEVEL=${PE_LEVEL} /opt/ibmhpc/${PE_LEVEL}/ppe.poe/bin/pe_install" 

  STDOUT "SSH: Removing temporary tar file from $MIC"
  ssh root@$MIC "rm -rf /tmp/$PERTE_TAR_FN"

  # work around for current MIC image
  STDOUT "SSH: Check /var/lock/subsys"
  ssh root@$MIC "if [[ ! -d /var/lock/subsys ]]; then mkdir -p /var/lock/subsys; fi"

  echo 
  echo
done
